#!/usr/local/bin/ibsh -f
###########################################################################
#

set device [gpib find voltmeter ]

gpib sic   $device
gpib ren   $device 1
gpib clear $device

set refresh 500
set block 0
gpib write $device "D1Ready"
gpib write $device "D0L0Q1OT2"



set function "VD"
set oldfunc $function
set unit "V"
set rate "RATE M"
set oldrate "RATE S"
set trigger "TRIGGER 1"
set oldtrigger "TRIGGER 2"

proc displ_refresh {} {
	global	result device refresh block unit \
		function oldfunc rate oldrate trigger oldtrigger

	if {$oldfunc != $function} {
		set oldfunc $function
		gpib write $device $function
		}
	if {$oldrate != $rate} {
		set oldrate $rate
		gpib write $device $rate
		}
	if {$oldtrigger != $trigger} {
		set oldtrigger $trigger
		gpib write $device $trigger
		}

	switch $function {
		"VD" {set unit "V"}
		"VA" {set unit "V"}
		"ID" {set unit "A"}
		"IA" {set unit "A"}
		"O2" {set unit "Ohm"}
#		"DIODE" {set unit "V"}
#		"FREQ" {set unit "Hz"}
#		"VACDC" {set unit "V"}
#		"AACDC" {set unit "A"}
		}


	gpib wait $device srq
        set result [string trim [gpib read $device 12] \x0a]
	if { $result == "ERROR" } {
		set unit ""
	} elseif {[eval string last E-3 $result] > 0} {
			set result [eval string trim $result E-3]
			set unit [format " m$unit"]
	} elseif {[eval string last E-6 $result] > 0} {
			set result [eval string trim $result E-3]
			set unit [format " u$unit"]
	} elseif {[eval string last E+0 $result] > 0} {
			set result [eval string trim $result E+0]
			set unit [format " $unit"]
	} elseif {[eval string last E+3 $result] > 0} {
			set result [eval string trim $result E+3]
			set unit [format " k$unit"]
	} elseif {[eval string last E+6 $result] > 0} {
			set result [eval string trim $result E+6]
			set unit [format " M$unit"]
	} elseif {[eval string last 1E+9 $result] > 0} {
			set result " "
			set unit [format "overflow"]
	} else {set unit [format " $unit"]
	}
	if { $block == 0 } { 
		after $refresh displ_refresh
        }
	update idletasks
}

frame .mbar -relief raised -bd 2
frame .body
pack .mbar .body -fill x

frame .body.display -relief sunken -borderwidth 4
frame .body.status
pack .body.status -side left
pack .body.display -side left -pady 20 -padx 20

label .body.display.val -textvariable result -width 11 \
	-font -adobe-helvetica-bold-r-normal-*-24-*-*-*-*-*-*-*
label .body.display.unit -textvariable unit \
	-font -adobe-helvetica-bold-r-normal-*-24-*-*-*-*-*-*-*
pack .body.display.unit .body.display.val -side right

label .body.status.func -textvariable function \
	-font -adobe-helvetica-bold-r-normal-*-10-*-*-*-*-*-*-*
label .body.status.rate -textvariable rate \
	-font -adobe-helvetica-bold-r-normal-*-10-*-*-*-*-*-*-*
label .body.status.trig -textvariable trigger \
	-font -adobe-helvetica-bold-r-normal-*-10-*-*-*-*-*-*-*
pack .body.status.func .body.status.rate .body.status.trig

menubutton .mbar.function -text Function -underline 0 \
	-menu .mbar.function.menu
menubutton .mbar.rate -text Rate -underline 0 \
	-menu .mbar.rate.menu
menubutton .mbar.trigger -text Trigger -underline 0 \
	-menu .mbar.trigger.menu
menubutton .mbar.misc -text Miscellaneous -underline 0 \
	-menu .mbar.misc.menu
pack .mbar.function .mbar.rate .mbar.trigger .mbar.misc -side left

menu .mbar.function.menu
.mbar.function.menu add radiobutton -label "V =" \
	-variable function -value "VD"
.mbar.function.menu add radiobutton -label "V ~" \
	-variable function -value "VA"
.mbar.function.menu add radiobutton -label "A =" \
	-variable function -value "ID"
.mbar.function.menu add radiobutton -label "A ~" \
	-variable function -value "IA"
.mbar.function.menu add radiobutton -label "Ohm" \
	-variable function -value "O2"
.mbar.function.menu add radiobutton -label "Diode" \
	-variable function -value "DIODE"
.mbar.function.menu add radiobutton -label "Freq" \
	-variable function -value "FREQ"
.mbar.function.menu add radiobutton -label "V (AC/DC)" \
	-variable function -value "VACDC"
.mbar.function.menu add radiobutton -label "A (AC/DC)" \
	-variable function -value "AACDC"
.mbar.function.menu add separator
.mbar.function.menu add command -label "Quit" \
	-command { set block 1; update idletasks; gpib close $device; exit}

menu .mbar.rate.menu
foreach val {S M F} {
	.mbar.rate.menu add radiobutton -label $val \
		-variable rate -value "RATE $val"
	}

menu .mbar.trigger.menu
foreach val {1 2 3 4 5} {
	.mbar.trigger.menu add radiobutton -label $val \
	 -variable trigger -value "TRIGGER $val"
	}

menu .mbar.misc.menu
foreach val {*CLS *OPC *RST *TRG *TST} {
	.mbar.misc.menu add command -label $val \
		-command { gpib write $device $val }
	}

tk_menuBar .mbar .mbar.function .mbar.rate .mbar.trigger .mbar.misc
focus .mbar

displ_refresh










